@@ -198,12 +198,6 @@ void QMediaRecorder::setCaptureSession(QMediaCaptureSession *session)
     and reset when new location is set or new recording starts.
 */
 
-/*!
-    \qmlproperty bool QtMultimedia::MediaRecorder::isAvailable
-    \brief This property holds whether the recorder service is ready to use.
-
-    Returns \c true if media recorder service ready to use.
-*/
 /*!
     Returns \c true if media recorder service ready to use.
 */
@@ -230,6 +224,35 @@ void QMediaRecorder::setOutputLocation(const QUrl &location)
             emit errorOccurred(QMediaRecorder::LocationNotWritable, tr("Output location not writable"));
 }
 
+/*!
+    Set the output IO device for media content.
+
+    The \a device must have been opened in the \l{QIODevice::Write}{Write} or
+    \l{QIODevice::ReadWrite}{ReadWrite} modes before the recording starts.
+
+    The media recorder doesn't take ownership of the specified \a device.
+    If the recording has been started, the device must be kept alive and open until
+    the signal \c recorderStateChanged(StoppedState) is emitted.
+
+    \sa outputDevice()
+*/
+void QMediaRecorder::setOutputDevice(QIODevice *device)
+{
+    Q_D(QMediaRecorder);
+    d->control->setOutputDevice(device);
+}
+
+/*!
+    Returns the output IO device for media content.
+
+    \sa setOutputDevice()
+*/
+QIODevice *QMediaRecorder::outputDevice() const
+{
+    Q_D(const QMediaRecorder);
+    return d->control->outputDevice();
+}
+
 QUrl QMediaRecorder::actualLocation() const
 {
     Q_D(const QMediaRecorder);
