@@ -41,10 +41,11 @@ class QRhiWithThreadGuard : public QObject {
     Q_OBJECT
 public:
     QRhiWithThreadGuard(std::shared_ptr<QRhi> r, std::shared_ptr<AndroidTextureThread> t)
-        : m_rhi(std::move(r)), m_thread(std::move(t)) {}
+        : m_guardRhi(std::move(r)), m_thread(std::move(t)) {}
     ~QRhiWithThreadGuard();
+protected:
+    std::shared_ptr<QRhi> m_guardRhi;
 private:
-    std::shared_ptr<QRhi> m_rhi;
     std::shared_ptr<AndroidTextureThread> m_thread;
 };
 
@@ -56,7 +57,7 @@ public:
             std::shared_ptr<QRhi> rhi, std::shared_ptr<AndroidTextureThread> thread,
             std::unique_ptr<QRhiTexture> tex, const QSize &size)
         : QRhiWithThreadGuard(std::move(rhi), std::move(thread))
-          , QAbstractVideoBuffer(QVideoFrame::RhiTextureHandle, rhi.get())
+          , QAbstractVideoBuffer(QVideoFrame::RhiTextureHandle, m_guardRhi.get())
           , m_size(size)
           , m_tex(std::move(tex))
     {}
@@ -298,10 +299,11 @@ public:
     }
 
 public slots:
-    void onFrameAvailable()
+    void onFrameAvailable(quint64 index)
     {
-        // Check if 'm_surfaceTexture' is not reset because there can be pending frames in queue.
-        if (m_surfaceTexture) {
+        // Check if 'm_surfaceTexture' is not reset and if the current index is the same that
+        // was used for creating connection because there can be pending frames in queue.
+        if (m_surfaceTexture && m_surfaceTexture->index() == index) {
             m_surfaceTexture->updateTexImage();
             auto matrix = extTransformMatrix(m_surfaceTexture.get());
             auto tex = m_textureCopy->copyExternalTexture(m_size, matrix);
@@ -337,8 +339,9 @@ public slots:
         m_texture->create();
         m_surfaceTexture = std::make_unique<AndroidSurfaceTexture>(m_texture->nativeTexture().object);
         if (m_surfaceTexture->surfaceTexture()) {
+            const quint64 index = m_surfaceTexture->index();
             connect(m_surfaceTexture.get(), &AndroidSurfaceTexture::frameAvailable, this,
-                    &AndroidTextureThread::onFrameAvailable);
+                    [this, index] () { this->onFrameAvailable(index); });
 
             m_textureCopy = std::make_unique<TextureCopy>(m_rhi.get(), m_texture.get());
 
@@ -365,7 +368,7 @@ private:
 QRhiWithThreadGuard::~QRhiWithThreadGuard() {
     // It may happen that reseting m_rhi shared_ptr will delete it (if it is the last reference)
     // QRHI need to be deleted from the thread that created it.
-    QMetaObject::invokeMethod(m_thread.get(), [&]() {m_rhi.reset();}, Qt::BlockingQueuedConnection);
+    QMetaObject::invokeMethod(m_thread.get(), [&]() {m_guardRhi.reset();}, Qt::BlockingQueuedConnection);
 }
 
 QAndroidTextureVideoOutput::QAndroidTextureVideoOutput(QVideoSink *sink, QObject *parent)
@@ -453,11 +456,6 @@ void QAndroidTextureVideoOutput::reset()
     if (m_sink)
         m_sink->platformVideoSink()->setVideoFrame({});
     QMetaObject::invokeMethod(m_surfaceThread.get(), &AndroidTextureThread::clearSurfaceTexture);
-    // We are not allowed to work on the same thread after the reset
-    // as there may be pending frames and should not be handled.
-    startNewSurfaceThread();
-    QMetaObject::invokeMethod(m_surfaceThread.get(),
-        [&](){ m_surfaceThread->setFrameSize(m_nativeSize); }, Qt::BlockingQueuedConnection);
 }
 
 void QAndroidTextureVideoOutput::newFrame(const QVideoFrame &frame)
