@@ -26,10 +26,6 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#if QT_CONFIG(gstreamer_gl)
-#  include <gst/gl/gl.h>
-#endif
-
 static Q_LOGGING_CATEGORY(qLcMediaPlayer, "qt.multimedia.player")
 
 QT_BEGIN_NAMESPACE
@@ -78,25 +74,13 @@ QGstreamerMediaPlayer::TrackSelector &QGstreamerMediaPlayer::trackSelector(Track
     return ts;
 }
 
-void QGstreamerMediaPlayer::updateBufferProgress(float newProgress)
-{
-    if (qFuzzyIsNull(newProgress - m_bufferProgress))
-        return;
-
-    m_bufferProgress = newProgress;
-    bufferProgressChanged(m_bufferProgress);
-}
-
 void QGstreamerMediaPlayer::disconnectDecoderHandlers()
 {
     auto handlers = std::initializer_list<QGObjectHandlerScopedConnection *>{
-        &padAdded,    &padRemoved,   &sourceSetup,    &uridecodebinElementAdded,
-        &unknownType, &elementAdded, &elementRemoved,
+        &padAdded, &padRemoved, &sourceSetup, &elementAdded, &unknownType,
     };
     for (QGObjectHandlerScopedConnection *handler : handlers)
         handler->disconnect();
-
-    decodeBinQueues = 0;
 }
 
 QMaybe<QPlatformMediaPlayer *> QGstreamerMediaPlayer::create(QMediaPlayer *parent)
@@ -105,26 +89,35 @@ QMaybe<QPlatformMediaPlayer *> QGstreamerMediaPlayer::create(QMediaPlayer *paren
     if (!videoOutput)
         return videoOutput.error();
 
-    static const auto error =
-            qGstErrorMessageIfElementsNotAvailable("input-selector", "decodebin", "uridecodebin");
-    if (error)
-        return *error;
+    QGstElement videoInputSelector =
+            QGstElement::createFromFactory("input-selector", "videoInputSelector");
+    if (!videoInputSelector)
+        return errorMessageCannotFindElement("input-selector");
 
-    return new QGstreamerMediaPlayer(videoOutput.value(), parent);
+    QGstElement audioInputSelector =
+            QGstElement::createFromFactory("input-selector", "audioInputSelector");
+    if (!audioInputSelector)
+        return errorMessageCannotFindElement("input-selector");
+
+    QGstElement subTitleInputSelector =
+            QGstElement::createFromFactory("input-selector", "subTitleInputSelector");
+    if (!subTitleInputSelector)
+        return errorMessageCannotFindElement("input-selector");
+
+    return new QGstreamerMediaPlayer(videoOutput.value(), videoInputSelector, audioInputSelector,
+                                     subTitleInputSelector, parent);
 }
 
 QGstreamerMediaPlayer::QGstreamerMediaPlayer(QGstreamerVideoOutput *videoOutput,
+                                             QGstElement videoInputSelector,
+                                             QGstElement audioInputSelector,
+                                             QGstElement subTitleInputSelector,
                                              QMediaPlayer *parent)
     : QObject(parent),
       QPlatformMediaPlayer(parent),
-      trackSelectors{ {
-              { VideoStream,
-                QGstElement::createFromFactory("input-selector", "videoInputSelector") },
-              { AudioStream,
-                QGstElement::createFromFactory("input-selector", "audioInputSelector") },
-              { SubtitleStream,
-                QGstElement::createFromFactory("input-selector", "subTitleInputSelector") },
-      } },
+      trackSelectors{ { { VideoStream, videoInputSelector },
+                        { AudioStream, audioInputSelector },
+                        { SubtitleStream, subTitleInputSelector } } },
       playerPipeline(QGstPipeline::create("playerPipeline")),
       gstVideoOutput(videoOutput)
 {
@@ -136,6 +129,7 @@ QGstreamerMediaPlayer::QGstreamerMediaPlayer(QGstreamerVideoOutput *videoOutput,
     for (auto &ts : trackSelectors)
         playerPipeline.add(ts.selector);
 
+    playerPipeline.setState(GST_STATE_NULL);
     playerPipeline.installMessageFilter(static_cast<QGstreamerBusMessageFilter *>(this));
     playerPipeline.installMessageFilter(static_cast<QGstreamerSyncMessageFilter *>(this));
 
@@ -145,9 +139,7 @@ QGstreamerMediaPlayer::QGstreamerMediaPlayer(QGstreamerVideoOutput *videoOutput,
 
     gst_pipeline_use_clock(playerPipeline.pipeline(), systemClock.get());
 
-    connect(&positionUpdateTimer, &QTimer::timeout, this, [this] {
-        updatePositionFromPipeline();
-    });
+    connect(&positionUpdateTimer, &QTimer::timeout, this, &QGstreamerMediaPlayer::updatePosition);
 }
 
 QGstreamerMediaPlayer::~QGstreamerMediaPlayer()
@@ -155,42 +147,25 @@ QGstreamerMediaPlayer::~QGstreamerMediaPlayer()
     playerPipeline.removeMessageFilter(static_cast<QGstreamerBusMessageFilter *>(this));
     playerPipeline.removeMessageFilter(static_cast<QGstreamerSyncMessageFilter *>(this));
     playerPipeline.setStateSync(GST_STATE_NULL);
+    topology.free();
 }
 
-std::chrono::nanoseconds QGstreamerMediaPlayer::pipelinePosition() const
+qint64 QGstreamerMediaPlayer::position() const
 {
-    if (m_url.isEmpty())
-        return {};
+    if (playerPipeline.isNull() || m_url.isEmpty())
+        return 0;
 
-    Q_ASSERT(playerPipeline);
-    return playerPipeline.position();
-}
-
-void QGstreamerMediaPlayer::updatePositionFromPipeline()
-{
-    using namespace std::chrono;
-
-    positionChanged(round<milliseconds>(pipelinePosition()));
-}
-
-void QGstreamerMediaPlayer::updateDurationFromPipeline()
-{
-    std::chrono::milliseconds d = playerPipeline.durationInMs();
-    qCDebug(qLcMediaPlayer) << "updateDurationFromPipeline" << d;
-    if (d != m_duration) {
-        m_duration = d;
-        durationChanged(m_duration);
-    }
+    return playerPipeline.position()/1e6;
 }
 
 qint64 QGstreamerMediaPlayer::duration() const
 {
-    return m_duration.count();
+    return m_duration;
 }
 
 float QGstreamerMediaPlayer::bufferProgress() const
 {
-    return m_bufferProgress;
+    return m_bufferProgress/100.;
 }
 
 QMediaTimeRange QGstreamerMediaPlayer::availablePlaybackRanges() const
@@ -205,28 +180,18 @@ qreal QGstreamerMediaPlayer::playbackRate() const
 
 void QGstreamerMediaPlayer::setPlaybackRate(qreal rate)
 {
-    if (rate == m_rate)
-        return;
-
-    m_rate = rate;
-
-    playerPipeline.setPlaybackRate(rate);
-    playbackRateChanged(rate);
+    if (playerPipeline.setPlaybackRate(rate))
+        playbackRateChanged(rate);
 }
 
 void QGstreamerMediaPlayer::setPosition(qint64 pos)
 {
-    std::chrono::milliseconds posInMs{ pos };
-    setPosition(posInMs);
-}
-
-void QGstreamerMediaPlayer::setPosition(std::chrono::milliseconds pos)
-{
-    if (pos == playerPipeline.position())
+    qint64 currentPos = playerPipeline.position()/1e6;
+    if (pos == currentPos)
         return;
     playerPipeline.finishStateChange();
-    playerPipeline.setPosition(pos);
-    qCDebug(qLcMediaPlayer) << Q_FUNC_INFO << pos << playerPipeline.positionInMs();
+    playerPipeline.setPosition(pos*1e6);
+    qCDebug(qLcMediaPlayer) << Q_FUNC_INFO << pos << playerPipeline.position()/1e6;
     if (mediaStatus() == QMediaPlayer::EndOfMedia)
         mediaStatusChanged(QMediaPlayer::LoadedMedia);
     positionChanged(pos);
@@ -236,14 +201,12 @@ void QGstreamerMediaPlayer::play()
 {
     if (state() == QMediaPlayer::PlayingState || m_url.isEmpty())
         return;
-
-    if (state() != QMediaPlayer::PausedState)
-        resetCurrentLoop();
+    resetCurrentLoop();
 
     playerPipeline.setInStoppedState(false);
     if (mediaStatus() == QMediaPlayer::EndOfMedia) {
-        playerPipeline.setPosition({});
-        positionChanged(0);
+        playerPipeline.setPosition(0);
+        updatePosition();
     }
 
     qCDebug(qLcMediaPlayer) << "play().";
@@ -253,17 +216,13 @@ void QGstreamerMediaPlayer::play()
         // immediately, when they happen while paused.
         playerPipeline.flush();
         m_requiresSeekOnPlay = false;
-    } else {
-        // we get an assertion failure during instant playback rate changes
-        // https://gitlab.freedesktop.org/gstreamer/gstreamer/-/issues/3545
-        constexpr bool performInstantRateChange = false;
-        playerPipeline.applyPlaybackRate(/*instantRateChange=*/performInstantRateChange);
     }
     if (ret == GST_STATE_CHANGE_FAILURE)
         qCDebug(qLcMediaPlayer) << "Unable to set the pipeline to the playing state.";
-
+    if (mediaStatus() == QMediaPlayer::LoadedMedia)
+        mediaStatusChanged(QMediaPlayer::BufferedMedia);
+    emit stateChanged(QMediaPlayer::PlayingState);
     positionUpdateTimer.start(100);
-    stateChanged(QMediaPlayer::PlayingState);
 }
 
 void QGstreamerMediaPlayer::pause()
@@ -277,83 +236,53 @@ void QGstreamerMediaPlayer::pause()
         playerPipeline.setInStoppedState(false);
         playerPipeline.flush();
     }
-    int ret = playerPipeline.setStateSync(GST_STATE_PAUSED);
+    int ret = playerPipeline.setState(GST_STATE_PAUSED);
     if (ret == GST_STATE_CHANGE_FAILURE)
         qCDebug(qLcMediaPlayer) << "Unable to set the pipeline to the paused state.";
     if (mediaStatus() == QMediaPlayer::EndOfMedia) {
-        playerPipeline.setPosition({});
-        positionChanged(0);
-    } else {
-        updatePositionFromPipeline();
-    }
-    stateChanged(QMediaPlayer::PausedState);
-
-    if (m_bufferProgress > 0 || !canTrackProgress())
+        playerPipeline.setPosition(0);
         mediaStatusChanged(QMediaPlayer::BufferedMedia);
-    else
-        mediaStatusChanged(QMediaPlayer::BufferingMedia);
+    }
+    updatePosition();
+    emit stateChanged(QMediaPlayer::PausedState);
 }
 
 void QGstreamerMediaPlayer::stop()
 {
-    using namespace std::chrono_literals;
     if (state() == QMediaPlayer::StoppedState) {
         if (position() != 0) {
-            playerPipeline.setPosition({});
-            positionChanged(0ms);
-            mediaStatusChanged(QMediaPlayer::LoadedMedia);
+            playerPipeline.setPosition(0);
+            positionChanged(0);
         }
         return;
     }
     stopOrEOS(false);
 }
 
-const QGstPipeline &QGstreamerMediaPlayer::pipeline() const
+void *QGstreamerMediaPlayer::nativePipeline()
 {
-    return playerPipeline;
+    return playerPipeline.pipeline();
 }
 
 void QGstreamerMediaPlayer::stopOrEOS(bool eos)
 {
-    using namespace std::chrono_literals;
-
     positionUpdateTimer.stop();
     playerPipeline.setInStoppedState(true);
     bool ret = playerPipeline.setStateSync(GST_STATE_PAUSED);
     if (!ret)
         qCDebug(qLcMediaPlayer) << "Unable to set the pipeline to the stopped state.";
-    if (!eos) {
-        playerPipeline.setPosition(0ms);
-        positionChanged(0ms);
-    }
-    stateChanged(QMediaPlayer::StoppedState);
-    if (eos)
-        mediaStatusChanged(QMediaPlayer::EndOfMedia);
-    else
-        mediaStatusChanged(QMediaPlayer::LoadedMedia);
-    m_initialBufferProgressSent = false;
-    bufferProgressChanged(0.f);
-}
-
-void QGstreamerMediaPlayer::detectPipelineIsSeekable()
-{
-    qCDebug(qLcMediaPlayer) << "detectPipelineIsSeekable";
-    QGstQueryHandle query{
-        gst_query_new_seeking(GST_FORMAT_TIME),
-        QGstQueryHandle::HasRef,
-    };
-    gboolean canSeek = false;
-    if (gst_element_query(playerPipeline.element(), query.get())) {
-        gst_query_parse_seeking(query.get(), nullptr, &canSeek, nullptr, nullptr);
-        qCDebug(qLcMediaPlayer) << "    pipeline is seekable:" << canSeek;
-    } else {
-        qCWarning(qLcMediaPlayer) << "    query for seekable failed.";
-    }
-    seekableChanged(canSeek);
+    if (!eos)
+        playerPipeline.setPosition(0);
+    updatePosition();
+    emit stateChanged(QMediaPlayer::StoppedState);
+    mediaStatusChanged(eos ? QMediaPlayer::EndOfMedia : QMediaPlayer::LoadedMedia);
 }
 
 bool QGstreamerMediaPlayer::processBusMessage(const QGstreamerMessage &message)
 {
+    if (message.isNull())
+        return false;
+
     qCDebug(qLcMediaPlayer) << "received bus message:" << message;
 
     GstMessage* gm = message.message();
@@ -364,55 +293,34 @@ bool QGstreamerMediaPlayer::processBusMessage(const QGstreamerMessage &message)
         gst_message_parse_tag(gm, &tagList);
 
         qCDebug(qLcMediaPlayer) << "    Got tags: " << tagList.get();
-
-        QMediaMetaData originalMetaData = m_metaData;
-        extendMetaDataFromTagList(m_metaData, tagList);
-        if (originalMetaData != m_metaData)
-            metaDataChanged();
-
-        if (gstVideoOutput) {
-            QVariant rotation = m_metaData.value(QMediaMetaData::Orientation);
-            gstVideoOutput->setRotation(rotation.value<QtVideo::Rotation>());
-        }
+        auto metaData = QGstreamerMetaData::fromGstTagList(tagList.get());
+        for (auto k : metaData.keys())
+            m_metaData.insert(k, metaData.value(k));
         break;
     }
     case GST_MESSAGE_DURATION_CHANGED: {
-        if (!prerolling)
-            updateDurationFromPipeline();
-
+        qint64 d = playerPipeline.duration()/1e6;
+        qCDebug(qLcMediaPlayer) << "    duration changed message" << d;
+        if (d != m_duration) {
+            m_duration = d;
+            emit durationChanged(duration());
+        }
         return false;
     }
-    case GST_MESSAGE_EOS: {
-        positionChanged(m_duration);
+    case GST_MESSAGE_EOS:
         if (doLoop()) {
             setPosition(0);
             break;
         }
         stopOrEOS(true);
         break;
-    }
     case GST_MESSAGE_BUFFERING: {
+        qCDebug(qLcMediaPlayer) << "    buffering message";
         int progress = 0;
         gst_message_parse_buffering(gm, &progress);
-
-        qCDebug(qLcMediaPlayer) << "    buffering message: " << progress;
-
-        if (state() != QMediaPlayer::StoppedState && !prerolling) {
-            if (!m_initialBufferProgressSent) {
-                mediaStatusChanged(QMediaPlayer::BufferingMedia);
-                m_initialBufferProgressSent = true;
-            }
-
-            if (m_bufferProgress > 0 && progress == 0)
-                mediaStatusChanged(QMediaPlayer::StalledMedia);
-            else if (progress >= 50)
-                // QTBUG-124517: rethink buffering
-                mediaStatusChanged(QMediaPlayer::BufferedMedia);
-            else
-                mediaStatusChanged(QMediaPlayer::BufferingMedia);
-        }
-
-        updateBufferProgress(progress * 0.01);
+        m_bufferProgress = progress;
+        mediaStatusChanged(m_bufferProgress == 100 ? QMediaPlayer::BufferedMedia : QMediaPlayer::BufferingMedia);
+        emit bufferProgressChanged(m_bufferProgress/100.);
         break;
     }
     case GST_MESSAGE_STATE_CHANGED: {
@@ -424,81 +332,78 @@ bool QGstreamerMediaPlayer::processBusMessage(const QGstreamerMessage &message)
         GstState    pending;
 
         gst_message_parse_state_changed(gm, &oldState, &newState, &pending);
-        qCDebug(qLcMediaPlayer) << "    state changed message from"
-                                << QCompactGstMessageAdaptor(message);
+        qCDebug(qLcMediaPlayer) << "    state changed message from" << oldState << "to" << newState
+                                << pending;
 
         switch (newState) {
         case GST_STATE_VOID_PENDING:
         case GST_STATE_NULL:
         case GST_STATE_READY:
             break;
-        case GST_STATE_PAUSED: {
+        case GST_STATE_PAUSED:
+        {
             if (prerolling) {
                 qCDebug(qLcMediaPlayer) << "Preroll done, setting status to Loaded";
-                playerPipeline.dumpGraph("playerPipelinePrerollDone");
-
                 prerolling = false;
-                updateDurationFromPipeline();
+                GST_DEBUG_BIN_TO_DOT_FILE(playerPipeline.bin(), GST_DEBUG_GRAPH_SHOW_ALL, "playerPipeline");
+
+                qint64 d = playerPipeline.duration()/1e6;
+                if (d != m_duration) {
+                    m_duration = d;
+                    qCDebug(qLcMediaPlayer) << "    duration changed" << d;
+                    emit durationChanged(duration());
+                }
 
-                m_metaData.insert(QMediaMetaData::Duration, duration());
-                m_metaData.insert(QMediaMetaData::Url, m_url);
                 parseStreamsAndMetadata();
-                metaDataChanged();
 
-                tracksChanged();
+                emit tracksChanged();
                 mediaStatusChanged(QMediaPlayer::LoadedMedia);
 
-                if (!playerPipeline.inStoppedState()) {
-                    Q_ASSERT(!m_initialBufferProgressSent);
-
-                    bool immediatelySendBuffered = !canTrackProgress() || m_bufferProgress > 0;
-                    mediaStatusChanged(QMediaPlayer::BufferingMedia);
-                    m_initialBufferProgressSent = true;
-                    if (immediatelySendBuffered)
-                        mediaStatusChanged(QMediaPlayer::BufferedMedia);
+                GstQuery *query = gst_query_new_seeking(GST_FORMAT_TIME);
+                gboolean canSeek = false;
+                if (gst_element_query(playerPipeline.element(), query)) {
+                    gst_query_parse_seeking(query, nullptr, &canSeek, nullptr, nullptr);
+                    qCDebug(qLcMediaPlayer) << "    pipeline is seekable:" << canSeek;
+                } else {
+                    qCDebug(qLcMediaPlayer) << "    query for seekable failed.";
                 }
+                gst_query_unref(query);
+                seekableChanged(canSeek);
             }
 
             break;
         }
-        case GST_STATE_PLAYING: {
-            if (!m_initialBufferProgressSent) {
-                bool immediatelySendBuffered = !canTrackProgress() || m_bufferProgress > 0;
-                mediaStatusChanged(QMediaPlayer::BufferingMedia);
-                m_initialBufferProgressSent = true;
-                if (immediatelySendBuffered)
-                    mediaStatusChanged(QMediaPlayer::BufferedMedia);
-            }
+        case GST_STATE_PLAYING:
+            mediaStatusChanged(QMediaPlayer::BufferedMedia);
             break;
         }
-        }
         break;
     }
     case GST_MESSAGE_ERROR: {
-        qCDebug(qLcMediaPlayer) << "    error" << QCompactGstMessageAdaptor(message);
-
         QUniqueGErrorHandle err;
         QUniqueGStringHandle debug;
         gst_message_parse_error(gm, &err, &debug);
+        qCDebug(qLcMediaPlayer) << "    error" << err << debug;
+
         GQuark errorDomain = err.get()->domain;
         gint errorCode = err.get()->code;
 
         if (errorDomain == GST_STREAM_ERROR) {
             if (errorCode == GST_STREAM_ERROR_CODEC_NOT_FOUND)
-                error(QMediaPlayer::FormatError, tr("Cannot play stream of type: <unknown>"));
+                emit error(QMediaPlayer::FormatError, tr("Cannot play stream of type: <unknown>"));
             else {
-                error(QMediaPlayer::FormatError, QString::fromUtf8(err.get()->message));
+                emit error(QMediaPlayer::FormatError, QString::fromUtf8(err.get()->message));
             }
         } else if (errorDomain == GST_RESOURCE_ERROR) {
             if (errorCode == GST_RESOURCE_ERROR_NOT_FOUND) {
                 if (m_resourceErrorState != ResourceErrorState::ErrorReported) {
                     // gstreamer seems to deliver multiple GST_RESOURCE_ERROR_NOT_FOUND events
-                    error(QMediaPlayer::ResourceError, QString::fromUtf8(err.get()->message));
+                    emit error(QMediaPlayer::ResourceError, QString::fromUtf8(err.get()->message));
                     m_resourceErrorState = ResourceErrorState::ErrorReported;
                     m_url.clear();
                 }
             } else {
-                error(QMediaPlayer::ResourceError, QString::fromUtf8(err.get()->message));
+                emit error(QMediaPlayer::ResourceError, QString::fromUtf8(err.get()->message));
             }
         } else {
             playerPipeline.dumpGraph("error");
@@ -506,40 +411,40 @@ bool QGstreamerMediaPlayer::processBusMessage(const QGstreamerMessage &message)
         mediaStatusChanged(QMediaPlayer::InvalidMedia);
         break;
     }
-
-    case GST_MESSAGE_WARNING:
-        qCWarning(qLcMediaPlayer) << "Warning:" << QCompactGstMessageAdaptor(message);
+    case GST_MESSAGE_WARNING: {
+        QUniqueGErrorHandle err;
+        QUniqueGStringHandle debug;
+        gst_message_parse_warning (gm, &err, &debug);
+        qCWarning(qLcMediaPlayer) << "Warning:" << err;
         playerPipeline.dumpGraph("warning");
         break;
-
-    case GST_MESSAGE_INFO:
-        if (qLcMediaPlayer().isDebugEnabled())
-            qCDebug(qLcMediaPlayer) << "Info:" << QCompactGstMessageAdaptor(message);
+    }
+    case GST_MESSAGE_INFO: {
+        if (qLcMediaPlayer().isDebugEnabled()) {
+            QUniqueGErrorHandle err;
+            QUniqueGStringHandle debug;
+            gst_message_parse_info (gm, &err, &debug);
+            qCDebug(qLcMediaPlayer) << "Info:" << err;
+        }
         break;
-
+    }
     case GST_MESSAGE_SEGMENT_START: {
         qCDebug(qLcMediaPlayer) << "    segment start message, updating position";
-        QGstStructureView structure(gst_message_get_structure(gm));
+        QGstStructure structure(gst_message_get_structure(gm));
         auto p = structure["position"].toInt64();
         if (p) {
-            std::chrono::milliseconds position{
-                (*p) / 1000000,
-            };
-            positionChanged(position);
+            qint64 position = (*p)/1000000;
+            emit positionChanged(position);
         }
         break;
     }
     case GST_MESSAGE_ELEMENT: {
-        QGstStructureView structure(gst_message_get_structure(gm));
+        QGstStructure structure(gst_message_get_structure(gm));
         auto type = structure.name();
-        if (type == "stream-topology")
-            topology = structure.clone();
-
-        break;
-    }
-
-    case GST_MESSAGE_ASYNC_DONE: {
-        detectPipelineIsSeekable();
+        if (type == "stream-topology") {
+            topology.free();
+            topology = structure.copy();
+        }
         break;
     }
 
@@ -619,19 +524,19 @@ void QGstreamerMediaPlayer::decoderPadAdded(const QGstElement &src, const QGstPa
         if (streamType == VideoStream) {
             connectOutput(ts);
             ts.setActiveInputPad(sinkPad);
-            videoAvailableChanged(true);
+            emit videoAvailableChanged(true);
         }
         else if (streamType == AudioStream) {
             connectOutput(ts);
             ts.setActiveInputPad(sinkPad);
-            audioAvailableChanged(true);
+            emit audioAvailableChanged(true);
         }
     }
 
     if (!prerolling)
-        tracksChanged();
+        emit tracksChanged();
 
-    decoderOutputMap.emplace(pad, sinkPad);
+    decoderOutputMap.insert(pad.name(), sinkPad);
 }
 
 void QGstreamerMediaPlayer::decoderPadRemoved(const QGstElement &src, const QGstPad &pad)
@@ -640,11 +545,9 @@ void QGstreamerMediaPlayer::decoderPadRemoved(const QGstElement &src, const QGst
         return;
 
     qCDebug(qLcMediaPlayer) << "Removed pad" << pad.name() << "from" << src.name();
-
-    auto it = decoderOutputMap.find(pad);
-    if (it == decoderOutputMap.end())
+    auto track = decoderOutputMap.value(pad.name());
+    if (track.isNull())
         return;
-    QGstPad track = it->second;
 
     auto ts = std::find_if(std::begin(trackSelectors), std::end(trackSelectors),
                            [&](TrackSelector &ts){ return ts.selector == track.parent(); });
@@ -701,7 +604,7 @@ void QGstreamerMediaPlayer::connectOutput(TrackSelector &ts)
         qCDebug(qLcMediaPlayer) << "connecting output for track type" << ts.type;
         playerPipeline.add(e);
         qLinkGstElements(ts.selector, e);
-        e.syncStateWithParent();
+        e.setState(GST_STATE_PAUSED);
     }
 
     ts.isConnected = true;
@@ -736,18 +639,6 @@ void QGstreamerMediaPlayer::removeOutput(TrackSelector &ts)
     ts.isConnected = false;
 }
 
-void QGstreamerMediaPlayer::removeDynamicPipelineElements()
-{
-    for (QGstElement *element : { &src, &decoder }) {
-        if (element->isNull())
-            continue;
-
-        element->setStateSync(GstState::GST_STATE_NULL);
-        playerPipeline.remove(*element);
-        *element = QGstElement{};
-    }
-}
-
 void QGstreamerMediaPlayer::uridecodebinElementAddedCallback(GstElement * /*uridecodebin*/,
                                                              GstElement *child,
                                                              QGstreamerMediaPlayer *)
@@ -756,7 +647,9 @@ void QGstreamerMediaPlayer::uridecodebinElementAddedCallback(GstElement * /*urid
     qCDebug(qLcMediaPlayer) << "New element added to uridecodebin:" << c.name();
 
     static const GType decodeBinType = [] {
-        QGstElementFactoryHandle factory = QGstElement::findFactory("decodebin");
+        QGstElementFactoryHandle factory = QGstElementFactoryHandle{
+            gst_element_factory_find("decodebin"),
+        };
         return gst_element_factory_get_element_type(factory.get());
     }();
 
@@ -812,43 +705,8 @@ void QGstreamerMediaPlayer::unknownTypeCallback(GstElement *decodebin, GstPad *p
     });
 }
 
-static bool isQueue(const QGstElement &element)
-{
-    static const GType queueType = [] {
-        QGstElementFactoryHandle factory = QGstElement::findFactory("queue");
-        return gst_element_factory_get_element_type(factory.get());
-    }();
-
-    static const GType multiQueueType = [] {
-        QGstElementFactoryHandle factory = QGstElement::findFactory("multiqueue");
-        return gst_element_factory_get_element_type(factory.get());
-    }();
-
-    return element.type() == queueType || element.type() == multiQueueType;
-}
-
-void QGstreamerMediaPlayer::decodebinElementAddedCallback(GstBin * /*decodebin*/,
-                                                          GstBin * /*sub_bin*/, GstElement *child,
-                                                          QGstreamerMediaPlayer *self)
-{
-    QGstElement c(child, QGstElement::NeedsRef);
-    if (isQueue(c))
-        self->decodeBinQueues += 1;
-}
-
-void QGstreamerMediaPlayer::decodebinElementRemovedCallback(GstBin * /*decodebin*/,
-                                                            GstBin * /*sub_bin*/, GstElement *child,
-                                                            QGstreamerMediaPlayer *self)
-{
-    QGstElement c(child, QGstElement::NeedsRef);
-    if (isQueue(c))
-        self->decodeBinQueues -= 1;
-}
-
 void QGstreamerMediaPlayer::setMedia(const QUrl &content, QIODevice *stream)
 {
-    using namespace std::chrono_literals;
-
     qCDebug(qLcMediaPlayer) << Q_FUNC_INFO << "setting location to" << content;
 
     prerolling = true;
@@ -861,27 +719,30 @@ void QGstreamerMediaPlayer::setMedia(const QUrl &content, QIODevice *stream)
     m_url = content;
     m_stream = stream;
 
-    removeDynamicPipelineElements();
+    if (!src.isNull())
+        playerPipeline.remove(src);
+    if (!decoder.isNull())
+        playerPipeline.remove(decoder);
+    src = QGstElement();
     disconnectDecoderHandlers();
+    decoder = QGstElement();
     removeAllOutputs();
     seekableChanged(false);
     Q_ASSERT(playerPipeline.inStoppedState());
 
-    if (m_duration != 0ms) {
-        m_duration = 0ms;
-        durationChanged(0ms);
+    if (m_duration != 0) {
+        m_duration = 0;
+        durationChanged(0);
     }
     stateChanged(QMediaPlayer::StoppedState);
     if (position() != 0)
-        positionChanged(0ms);
+        positionChanged(0);
+    mediaStatusChanged(QMediaPlayer::NoMedia);
     if (!m_metaData.isEmpty()) {
         m_metaData.clear();
         metaDataChanged();
     }
 
-    if (content.isEmpty() && !stream)
-        mediaStatusChanged(QMediaPlayer::NoMedia);
-
     if (content.isEmpty())
         return;
 
@@ -891,23 +752,20 @@ void QGstreamerMediaPlayer::setMedia(const QUrl &content, QIODevice *stream)
             if (maybeAppSrc) {
                 m_appSrc = maybeAppSrc.value();
             } else {
-                error(QMediaPlayer::ResourceError, maybeAppSrc.error());
+                emit error(QMediaPlayer::ResourceError, maybeAppSrc.error());
                 return;
             }
         }
         src = m_appSrc->element();
         decoder = QGstElement::createFromFactory("decodebin", "decoder");
         if (!decoder) {
-            error(QMediaPlayer::ResourceError, qGstErrorMessageCannotFindElement("decodebin"));
+            emit error(QMediaPlayer::ResourceError, errorMessageCannotFindElement("decodebin"));
             return;
         }
         decoder.set("post-stream-topology", true);
         decoder.set("use-buffering", true);
-        unknownType = decoder.connect("unknown-type", GCallback(unknownTypeCallback), this);
-        elementAdded = decoder.connect("deep-element-added",
-                                       GCallback(decodebinElementAddedCallback), this);
-        elementRemoved = decoder.connect("deep-element-removed",
-                                         GCallback(decodebinElementAddedCallback), this);
+        unknownType = decoder.connect("unknown-type",
+                                      GCallback(QGstreamerMediaPlayer::unknownTypeCallback), this);
 
         playerPipeline.add(src, decoder);
         qLinkGstElements(src, decoder);
@@ -918,7 +776,7 @@ void QGstreamerMediaPlayer::setMedia(const QUrl &content, QIODevice *stream)
         // use uridecodebin
         decoder = QGstElement::createFromFactory("uridecodebin", "decoder");
         if (!decoder) {
-            error(QMediaPlayer::ResourceError, qGstErrorMessageCannotFindElement("uridecodebin"));
+            emit error(QMediaPlayer::ResourceError, errorMessageCannotFindElement("uridecodebin"));
             return;
         }
         playerPipeline.add(decoder);
@@ -929,38 +787,34 @@ void QGstreamerMediaPlayer::setMedia(const QUrl &content, QIODevice *stream)
         } else {
             // can't set post-stream-topology to true, as uridecodebin doesn't have the property.
             // Use a hack
-            uridecodebinElementAdded = decoder.connect(
-                    "element-added", GCallback(uridecodebinElementAddedCallback), this);
+            elementAdded = decoder.connect(
+                    "element-added",
+                    GCallback(QGstreamerMediaPlayer::uridecodebinElementAddedCallback), this);
         }
 
-        sourceSetup = decoder.connect("source-setup", GCallback(sourceSetupCallback), this);
-        unknownType = decoder.connect("unknown-type", GCallback(unknownTypeCallback), this);
+        sourceSetup = decoder.connect("source-setup",
+                                      GCallback(QGstreamerMediaPlayer::sourceSetupCallback), this);
+
+        unknownType = decoder.connect("unknown-type",
+                                      GCallback(QGstreamerMediaPlayer::unknownTypeCallback), this);
 
         decoder.set("uri", content.toEncoded().constData());
         decoder.set("use-buffering", true);
-
-        constexpr int mb = 1024 * 1024;
-        decoder.set("ring-buffer-max-size", 2 * mb);
-
-        updateBufferProgress(0.f);
-
-        elementAdded = decoder.connect("deep-element-added",
-                                       GCallback(decodebinElementAddedCallback), this);
-        elementRemoved = decoder.connect("deep-element-removed",
-                                         GCallback(decodebinElementAddedCallback), this);
+        if (m_bufferProgress != 0) {
+            m_bufferProgress = 0;
+            emit bufferProgressChanged(0.);
+        }
     }
     padAdded = decoder.onPadAdded<&QGstreamerMediaPlayer::decoderPadAdded>(this);
     padRemoved = decoder.onPadRemoved<&QGstreamerMediaPlayer::decoderPadRemoved>(this);
 
     mediaStatusChanged(QMediaPlayer::LoadingMedia);
-    if (!playerPipeline.setStateSync(GST_STATE_PAUSED)) {
-        qCWarning(qLcMediaPlayer) << "Unable to set the pipeline to the paused state.";
-        // Note: no further error handling: errors will be delivered via a GstMessage
-        return;
-    }
 
-    playerPipeline.setPosition(0ms);
-    positionChanged(0ms);
+    if (!playerPipeline.setState(GST_STATE_PAUSED))
+        qCWarning(qLcMediaPlayer) << "Unable to set the pipeline to the paused state.";
+
+    playerPipeline.setPosition(0);
+    positionChanged(0);
 }
 
 void QGstreamerMediaPlayer::setAudioOutput(QPlatformAudioOutput *output)
@@ -990,9 +844,9 @@ void QGstreamerMediaPlayer::setVideoSink(QVideoSink *sink)
     gstVideoOutput->setVideoSink(sink);
 }
 
-static QGstStructureView endOfChain(const QGstStructureView &s)
+static QGstStructure endOfChain(const QGstStructure &s)
 {
-    QGstStructureView e = s;
+    QGstStructure e = s;
     while (1) {
         auto next = e["next"].toStructure();
         if (!next.isNull())
@@ -1006,26 +860,32 @@ static QGstStructureView endOfChain(const QGstStructureView &s)
 void QGstreamerMediaPlayer::parseStreamsAndMetadata()
 {
     qCDebug(qLcMediaPlayer) << "============== parse topology ============";
-
-    if (!topology) {
+    if (topology.isNull()) {
         qCDebug(qLcMediaPlayer) << "    null topology";
         return;
     }
+    auto caps = topology["caps"].toCaps();
+    auto structure = caps.at(0);
+    auto fileFormat = QGstreamerFormatInfo::fileFormatForCaps(structure);
+    qCDebug(qLcMediaPlayer) << caps << fileFormat;
+    m_metaData.insert(QMediaMetaData::FileFormat, QVariant::fromValue(fileFormat));
+    m_metaData.insert(QMediaMetaData::Duration, duration());
+    m_metaData.insert(QMediaMetaData::Url, m_url);
+    QGValue tags = topology["tags"];
+    if (!tags.isNull()) {
+        QGstTagListHandle tagList;
+        gst_structure_get(topology.structure, "tags", GST_TYPE_TAG_LIST, &tagList, nullptr);
 
-    QGstStructureView topologyView{ topology };
+        const auto metaData = QGstreamerMetaData::fromGstTagList(tagList.get());
+        for (auto k : metaData.keys())
+            m_metaData.insert(k, metaData.value(k));
+    }
 
-    QGstCaps caps = topologyView.caps();
-    extendMetaDataFromCaps(m_metaData, caps);
-
-    QGstTagListHandle tagList = QGstStructureView{ topology }.tags();
-    if (tagList)
-        extendMetaDataFromTagList(m_metaData, tagList);
-
-    QGstStructureView demux = endOfChain(topologyView);
-    QGValue next = demux["next"];
+    auto demux = endOfChain(topology);
+    auto next = demux["next"];
     if (!next.isList()) {
         qCDebug(qLcMediaPlayer) << "    no additional streams";
-        metaDataChanged();
+        emit metaDataChanged();
         return;
     }
 
@@ -1033,28 +893,43 @@ void QGstreamerMediaPlayer::parseStreamsAndMetadata()
     int size = next.listSize();
     for (int i = 0; i < size; ++i) {
         auto val = next.at(i);
-        caps = val.toStructure().caps();
+        caps = val.toStructure()["caps"].toCaps();
+        structure = caps.at(0);
+        if (structure.name().startsWith("audio/")) {
+            auto codec = QGstreamerFormatInfo::audioCodecForCaps(structure);
+            m_metaData.insert(QMediaMetaData::AudioCodec, QVariant::fromValue(codec));
+            qCDebug(qLcMediaPlayer) << "    audio" << caps << (int)codec;
+        } else if (structure.name().startsWith("video/")) {
+            auto codec = QGstreamerFormatInfo::videoCodecForCaps(structure);
+            m_metaData.insert(QMediaMetaData::VideoCodec, QVariant::fromValue(codec));
+            qCDebug(qLcMediaPlayer) << "    video" << caps << (int)codec;
+            auto framerate = structure["framerate"].getFraction();
+            if (framerate)
+                m_metaData.insert(QMediaMetaData::VideoFrameRate, *framerate);
 
-        extendMetaDataFromCaps(m_metaData, caps);
-
-        QGstStructureView structure = caps.at(0);
-
-        if (structure.name().startsWith("video/")) {
-            QSize nativeSize = structure.nativeSize();
-            gstVideoOutput->setNativeSize(nativeSize);
+            QSize resolution = structure.resolution();
+            if (resolution.isValid())
+                m_metaData.insert(QMediaMetaData::Resolution, resolution);
         }
     }
 
     auto sinkPad = trackSelector(VideoStream).activeInputPad();
-    if (sinkPad) {
-        QGstTagListHandle tagList = sinkPad.tags();
+    if (!sinkPad.isNull()) {
+        QGstTagListHandle tagList;
+
+        g_object_get(sinkPad.object(), "tags", &tagList, nullptr);
         if (tagList)
             qCDebug(qLcMediaPlayer) << "    tags=" << tagList.get();
         else
             qCDebug(qLcMediaPlayer) << "    tags=(null)";
+
+        QSize nativeSize = structure.nativeSize();
+        gstVideoOutput->setNativeSize(nativeSize);
     }
 
+
     qCDebug(qLcMediaPlayer) << "============== end parse topology ============";
+    emit metaDataChanged();
     playerPipeline.dumpGraph("playback");
 }
 
@@ -1066,11 +941,13 @@ int QGstreamerMediaPlayer::trackCount(QPlatformMediaPlayer::TrackType type)
 QMediaMetaData QGstreamerMediaPlayer::trackMetaData(QPlatformMediaPlayer::TrackType type, int index)
 {
     auto track = trackSelector(type).inputPad(index);
-    if (!track)
+    if (track.isNull())
         return {};
 
-    QGstTagListHandle tagList = track.tags();
-    return taglistToMetaData(tagList);
+    QGstTagListHandle tagList;
+    g_object_get(track.object(), "tags", &tagList, nullptr);
+
+    return tagList ? QGstreamerMetaData::fromGstTagList(tagList.get()) : QMediaMetaData{};
 }
 
 int QGstreamerMediaPlayer::activeTrack(TrackType type)
@@ -1109,3 +986,5 @@ void QGstreamerMediaPlayer::setActiveTrack(TrackType type, int index)
 }
 
 QT_END_NAMESPACE
+
+#include "moc_qgstreamermediaplayer_p.cpp"
