@@ -216,8 +216,8 @@ void tst_QCameraBackend::testCameraActive()
     session.setCamera(&camera);
     session.setImageCapture(&imageCapture);
 
-    QSignalSpy errorSignal(&camera, &QCamera::errorOccurred);
-    QSignalSpy activeChangedSignal(&camera, &QCamera::activeChanged);
+    QSignalSpy errorSignal(&camera, SIGNAL(errorOccurred(QCamera::Error, const QString &)));
+    QSignalSpy activeChangedSignal(&camera, SIGNAL(activeChanged(bool)));
 
     QCOMPARE(camera.isActive(), false);
 
@@ -280,7 +280,7 @@ void tst_QCameraBackend::testCameraFormat()
     if (videoFormats.isEmpty())
         QSKIP("No Camera available, skipping test.");
     QCameraFormat cameraFormat = videoFormats.first();
-    QSignalSpy spy(&camera, &QCamera::cameraFormatChanged);
+    QSignalSpy spy(&camera, SIGNAL(cameraFormatChanged()));
     QVERIFY(spy.size() == 0);
 
     QMediaCaptureSession session;
@@ -344,9 +344,9 @@ void tst_QCameraBackend::testCameraCapture()
 
     QVERIFY(!imageCapture.isReadyForCapture());
 
-    QSignalSpy capturedSignal(&imageCapture, &QImageCapture::imageCaptured);
-    QSignalSpy savedSignal(&imageCapture, &QImageCapture::imageSaved);
-    QSignalSpy errorSignal(&imageCapture, &QImageCapture::errorOccurred);
+    QSignalSpy capturedSignal(&imageCapture, SIGNAL(imageCaptured(int,QImage)));
+    QSignalSpy savedSignal(&imageCapture, SIGNAL(imageSaved(int,QString)));
+    QSignalSpy errorSignal(&imageCapture, SIGNAL(errorOccurred(int,QImageCapture::Error,const QString&)));
 
     imageCapture.captureToFile();
     QTRY_COMPARE(errorSignal.size(), 1);
@@ -367,7 +367,7 @@ void tst_QCameraBackend::testCameraCapture()
 
     int id = imageCapture.captureToFile();
 
-    QTRY_VERIFY_WITH_TIMEOUT(!savedSignal.isEmpty(), 8000);
+    QTRY_VERIFY_WITH_TIMEOUT(!savedSignal.isEmpty(), 8s);
 
     QTRY_COMPARE(capturedSignal.size(), 1);
     QCOMPARE(capturedSignal.last().first().toInt(), id);
@@ -403,10 +403,10 @@ void tst_QCameraBackend::testCaptureToBuffer()
 
     QTRY_VERIFY(camera.isActive());
 
-    QSignalSpy capturedSignal(&imageCapture, &QImageCapture::imageCaptured);
-    QSignalSpy imageAvailableSignal(&imageCapture, &QImageCapture::imageAvailable);
-    QSignalSpy savedSignal(&imageCapture, &QImageCapture::imageSaved);
-    QSignalSpy errorSignal(&imageCapture, &QImageCapture::errorOccurred);
+    QSignalSpy capturedSignal(&imageCapture, SIGNAL(imageCaptured(int,QImage)));
+    QSignalSpy imageAvailableSignal(&imageCapture, SIGNAL(imageAvailable(int,QVideoFrame)));
+    QSignalSpy savedSignal(&imageCapture, SIGNAL(imageSaved(int,QString)));
+    QSignalSpy errorSignal(&imageCapture, SIGNAL(errorOccurred(int,QImageCapture::Error,const QString&)));
 
     camera.start();
     QTRY_VERIFY(imageCapture.isReadyForCapture());
@@ -448,14 +448,8 @@ void tst_QCameraBackend::testCameraCaptureMetadata()
 
     camera.setFlashMode(QCamera::FlashOff);
 
-    QMediaMetaData referenceMetaData;
-    referenceMetaData.insert(QMediaMetaData::Title, QStringLiteral("Title"));
-    referenceMetaData.insert(QMediaMetaData::Language, QVariant::fromValue(QLocale::German));
-    referenceMetaData.insert(QMediaMetaData::Description, QStringLiteral("Description"));
-    imageCapture.setMetaData(referenceMetaData);
-
-    QSignalSpy metadataSignal(&imageCapture, &QImageCapture::imageMetadataAvailable);
-    QSignalSpy savedSignal(&imageCapture, &QImageCapture::imageSaved);
+    QSignalSpy metadataSignal(&imageCapture, SIGNAL(imageMetadataAvailable(int,const QMediaMetaData&)));
+    QSignalSpy savedSignal(&imageCapture, SIGNAL(imageSaved(int,QString)));
 
     camera.start();
 
@@ -466,19 +460,7 @@ void tst_QCameraBackend::testCameraCaptureMetadata()
     int id = imageCapture.captureToFile(tmpFile);
     QTRY_VERIFY(!savedSignal.isEmpty());
     QVERIFY(!metadataSignal.isEmpty());
-
     QCOMPARE(metadataSignal.first().first().toInt(), id);
-    QMediaMetaData receivedMetaData = metadataSignal.first()[1].value<QMediaMetaData>();
-
-    if (isGStreamerPlatform()) {
-        for (auto key : {
-                     QMediaMetaData::Title,
-                     QMediaMetaData::Language,
-                     QMediaMetaData::Description,
-             })
-            QCOMPARE(receivedMetaData[key], referenceMetaData[key]);
-        QVERIFY(receivedMetaData[QMediaMetaData::Resolution].isValid());
-    }
 }
 
 void tst_QCameraBackend::testExposureCompensation()
@@ -490,9 +472,9 @@ void tst_QCameraBackend::testExposureCompensation()
     QCamera camera;
     session.setCamera(&camera);
 
-    QSignalSpy exposureCompensationSignal(&camera, &QCamera::exposureCompensationChanged);
+    QSignalSpy exposureCompensationSignal(&camera, SIGNAL(exposureCompensationChanged(float)));
 
-    // it should be possible to set exposure parameters in Unloaded state
+    //it should be possible to set exposure parameters in Unloaded state
     QCOMPARE(camera.exposureCompensation(), 0.);
     if (!(camera.supportedFeatures() & QCamera::Feature::ExposureCompensation))
         return;
@@ -589,10 +571,10 @@ void tst_QCameraBackend::testVideoRecording()
     QMediaRecorder recorder;
     session.setRecorder(&recorder);
 
-    QSignalSpy errorSignal(camera.data(), &QCamera::errorOccurred);
-    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
-    QSignalSpy recorderStateChanged(&recorder, &QMediaRecorder::recorderStateChanged);
-    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
+    QSignalSpy errorSignal(camera.data(), SIGNAL(errorOccurred(QCamera::Error, const QString &)));
+    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(QMediaRecorder::Error, const QString &)));
+    QSignalSpy recorderStateChanged(&recorder, SIGNAL(recorderStateChanged(RecorderState)));
+    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
 
     recorder.setVideoResolution(320, 240);
 
@@ -638,7 +620,7 @@ void tst_QCameraBackend::testVideoRecording()
     QMediaPlayer player;
     player.setSource(fileName);
 
-    QTRY_COMPARE_WITH_TIMEOUT(player.mediaStatus(), QMediaPlayer::LoadedMedia, 8000);
+    QTRY_COMPARE_WITH_TIMEOUT(player.mediaStatus(), QMediaPlayer::LoadedMedia, 8s);
     QCOMPARE_EQ(player.metaData().value(QMediaMetaData::Resolution).toSize(), QSize(320, 240));
     QCOMPARE_GT(player.duration(), 350);
     QCOMPARE_LT(player.duration(), 650);
@@ -661,10 +643,10 @@ void tst_QCameraBackend::testNativeMetadata()
     QMediaRecorder recorder;
     session.setRecorder(&recorder);
 
-    QSignalSpy errorSignal(&camera, &QCamera::errorOccurred);
-    QSignalSpy recorderErrorSignal(&recorder, &QMediaRecorder::errorOccurred);
-    QSignalSpy recorderStateChanged(&recorder, &QMediaRecorder::recorderStateChanged);
-    QSignalSpy durationChanged(&recorder, &QMediaRecorder::durationChanged);
+    QSignalSpy errorSignal(&camera, SIGNAL(errorOccurred(QCamera::Error, const QString &)));
+    QSignalSpy recorderErrorSignal(&recorder, SIGNAL(errorOccurred(Error, const QString &)));
+    QSignalSpy recorderStateChanged(&recorder, SIGNAL(recorderStateChanged(RecorderState)));
+    QSignalSpy durationChanged(&recorder, SIGNAL(durationChanged(qint64)));
 
     camera.start();
     if (device.isNull()) {
@@ -705,6 +687,8 @@ void tst_QCameraBackend::testNativeMetadata()
     QVERIFY(!fileName.isEmpty());
     QVERIFY(QFileInfo(fileName).size() > 0);
 
+    QSKIP_GSTREAMER("QTBUG-124182: spurious failure while retrieving the metadata");
+
     // QMediaRecorder::metaData() can only test that QMediaMetaData is set properly on the recorder.
     // Use QMediaPlayer to test that the native metadata is properly set on the track
     QAudioOutput output;
@@ -716,20 +700,15 @@ void tst_QCameraBackend::testNativeMetadata()
     player.setSource(QUrl::fromLocalFile(fileName));
     player.play();
 
-    int metadataChangedRequiredCount = isGStreamerPlatform() ? 2 : 1;
+    QTRY_VERIFY(metadataChangedSpy.size() > 0);
 
-    QTRY_VERIFY(metadataChangedSpy.size() >= metadataChangedRequiredCount);
-
-    QCOMPARE(player.metaData().value(QMediaMetaData::Title).toString(),
-             metaData.value(QMediaMetaData::Title).toString());
+    QCOMPARE(player.metaData().value(QMediaMetaData::Title).toString(), metaData.value(QMediaMetaData::Title).toString());
     auto lang = player.metaData().value(QMediaMetaData::Language).value<QLocale::Language>();
     if (lang != QLocale::AnyLanguage)
         QCOMPARE(lang, metaData.value(QMediaMetaData::Language).value<QLocale::Language>());
     QCOMPARE(player.metaData().value(QMediaMetaData::Description).toString(), metaData.value(QMediaMetaData::Description).toString());
-    QVERIFY(player.metaData().value(QMediaMetaData::Resolution).isValid());
 
-    if (isGStreamerPlatform())
-        QVERIFY(player.metaData().value(QMediaMetaData::Date).isValid());
+    metadataChangedSpy.clear();
 
     player.stop();
     player.setSource({});
